#
# spec file for package include-what-you-use
#
# Copyright (c) 2018 SUSE LINUX GmbH, Nuernberg, Germany.
# Copyright (c) 2018 Aaron Puchert.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


Name:           include-what-you-use
Version:        0.9
Release:        lp150.2.1
Summary:        A tool to analyze #includes in C and C++ source files
License:        NCSA
Group:          Development/Languages/C and C++
Url:            https://include-what-you-use.org/
Source0:        https://include-what-you-use.org/downloads/%{name}-%{version}.src.tar.gz
Source1:        %{name}.1.gz
Patch0:         llvm-link.patch
Patch1:         fix-shebang.patch
Patch2:         iwyu_include_picker.patch
Patch3:         remove-x86-specific-code.patch
BuildRequires:  clang5
BuildRequires:  clang5-devel
BuildRequires:  cmake
%if 0%{?sle_version} && 0%{?sle_version} <= 130000 && !0%{?is_opensuse}
BuildRequires:  gcc6
BuildRequires:  gcc6-c++
%endif
BuildRequires:  llvm5-devel
# We need the compiler-specific headers in /usr/lib64/clang/.
Requires:       clang5
BuildRoot:      %{_tmppath}/%{name}-%{version}-build

%description
"Include what you use" means this: for every symbol (type, function,
variable, or macro) that you use in foo.cc (or foo.cpp), either foo.cc
or foo.h should include a .h file that exports the declaration of that
symbol. The include-what-you-use program is a tool to analyze includes
of source files to find include-what-you-use violations, and suggest
fixes for them.

The main goal of include-what-you-use is to remove superfluous includes.
It does this both by figuring out what includes are not actually needed
for this file (for both .cc and .h files), and replacing includes with
forward declarations when possible.

%package tools
Summary:        Additional tools to use %{name} effectively
Group:          Development/Languages/C and C++
Requires:       %{name} = %{version}-%{release}
BuildArch:      noarch

%description tools
This package contains additional scripts for using %{name} as automated
refactoring tool.

%prep
%setup -q -n %{name}
%patch0 -p1
%patch1 -p1
%patch2 -p1
%patch3 -p1

%build
# Remove obsolete files - this is now hardcoded into iwyu_include_picker.cc.
rm gcc.libc.imp gcc.symbols.imp gcc.stl.headers.imp stl.c.headers.imp
# The includes from third_party.imp probably only make sense in some code bases.
rm third_party.imp
# This also obsoletes iwyu.gcc.imp.
rm iwyu.gcc.imp

# Since Clang is built using Clang, use it here too. Except for SLES, where we
# need to use GCC 6 for some reason. (Otherwise there are segfaults.)
%if 0%{?sle_version} && 0%{?sle_version} <= 130000 && !0%{?is_opensuse}
export CC=gcc-6 CXX=g++-6
%else
export CC=clang CXX=clang++
%endif
%cmake -DIWYU_LLVM_ROOT_PATH=%{_libdir} ..
%make_jobs

%install
%cmake_install

install -Dm0644 %{SOURCE1} %{buildroot}%{_mandir}/man1/%{name}.1.gz

%files
%defattr(-, root, root, -)
%{_bindir}/%{name}
%{_mandir}/man1/%{name}.1.gz
%{_datadir}/%{name}/

%files tools
%defattr(-, root, root, -)
%{_bindir}/*.py

%changelog
* Sat Mar 17 2018 aaronpuchert@alice-dsl.net
- Make tools package "noarch", since it contains only scripts.
* Thu Mar 15 2018 aaronpuchert@alice-dsl.net
- Use tarball from website, where it is now available.
  There should be no functional change.
* Tue Feb 20 2018 aaronpuchert@alice-dsl.net
- Add runtime dependency to clang, because we need the compiler-
  specific headers, even when clang is not used for compilation.
* Tue Feb 20 2018 aaronpuchert@alice-dsl.net
- Fix build on SLE 12. For some reason the build using Clang
  segfaults, but with GCC 6 it doesn't.
- Use GCC 6 only on SLE 12, not on Leap 42.
* Mon Feb 19 2018 aaronpuchert@alice-dsl.net
- Add section about mapping files to the man page.
* Tue Feb  6 2018 aaronpuchert@alice-dsl.net
- Remove x86-specific code that is required to parse Microsoft
  inline assembly. We don't need that, and it breaks the builds
  on other architectures.
* Mon Feb  5 2018 aaronpuchert@alice-dsl.net
- Use tarball for version 0.9, which is now officially released.
- Add a manual page derived from the `--help` output.
* Sat Feb  3 2018 aaronpuchert@alice-dsl.net
- Update to development version of 0.9, which is not officially
  released yet.
- Update LLVM/Clang to version 5.
- Build using Clang instead of GCC, since we link against
  libclang's C++ API, which is also built with Clang.
- Also remove third_party.imp, which doesn't seem to be made for
  the general public.
- Package the Python tools separately.
- Improve description.
* Mon Dec 11 2017 aaronpuchert@alice-dsl.net
- Fix malformed patch.
* Mon Dec 11 2017 aaronpuchert@alice-dsl.net
- Update mappings in the right place - they are actually hard-coded
  into iwyu_include_picker.cc.
- Remove the obsolete mapping files.
* Sun Dec 10 2017 aaronpuchert@alice-dsl.net
- Initialized based on version 0.8, which uses Clang 4.0.
- Added llvm-link.patch because openSUSE's libLLVM isn't split up.
- Updated mappings for GNU libc headers in libc-mappings.patch.
- Updated mappings for GNU libstdc++ headers in stl-mappings.patch.
